#include <math.h>
#include "engine.h"
#pragma comment(lib,"opengl.lib")
#pragma comment(lib,"glut.lib")		

#define CUBE_SIZE 100
#define PI  3.1415926535897932384626433832795
#define EPS 0.001


vector_3d::vector_3d(double x0,double y0,double z0)
{
	x=x0;
	y=y0;
	z=z0;
}

void vector_3d::operator +=(vector_3d &obj)
{
	x+=obj.x; y+=obj.y; z+=obj.z;
}

void vector_3d::operator -=(vector_3d &obj)
{
	x-=obj.x; y-=obj.y; z-=obj.z;
}

void vector_3d::operator /=(double d)
{
	x/=d; y/=d; z/=d; 
}

vector_3d::operator double()
{
	return x*x+y*y+z*z;
}

double operator*(vector_3d &obj1,vector_3d &obj2)
{
	return obj1.x*obj2.x+obj1.y*obj2.y+obj1.z*obj2.z;
}

vector_3d operator -(vector_3d &obj1,vector_3d &obj2)
{
	vector_3d obj;
	obj.x=obj1.x-obj2.x;
	obj.y=obj1.y-obj2.y;
	obj.z=obj1.z-obj2.z;
	return obj;
}

vector_3d operator *(vector_3d &obj0,double d)
{
	vector_3d obj;
	obj.x=obj0.x*d;
	obj.y=obj0.y*d;
	obj.z=obj0.z*d;
	return obj;
}




CObj::CObj(COLORREF color0,vector_3d &offset0,int r0,vector_3d &speed0)
{
	r=r0;
	m=r0*r0*r0;
	color=color0;
	offset=offset0;
	speed=speed0;
	if(!color) {
		unsigned int r=rand()%100+150;
		unsigned int g=rand()%100+150;
		unsigned int b=rand()%100+150;
		color=RGB(r,g,b);
	}
}

bool CObj::isStand()
{
	return fabs(speed.x)<EPS && fabs(speed.y)<EPS && fabs(speed.z)<EPS;
}


Engine::Engine()
{
	f1=30.0; f2=70.0;
}

GLvoid Engine::Resize(GLsizei width, GLsizei height)
{
	size.cx=width;
	size.cy=height;
	glViewport(0, 0, width, height);					//   
// SetProjectionMatrix
	glMatrixMode(GL_PROJECTION);					//      
	glLoadIdentity();								//   ()   
	glFrustum(-50*size.cx/size.cy,50*size.cx/size.cy,-50,50,300,500); //   
// SetModelviewMatrix
	glMatrixMode(GL_MODELVIEW);							//      
	glLoadIdentity();									//   ()   
	glTranslatef(0.0, 0.0, -400.0);						//     8   													
}


GLvoid Engine::InitScene()
{
	int x,y,z;
	m_list.push_back(CObj(RGB(-1,-1,-1),vector_3d(-45,-41,-43),4,vector_3d(1,1,1)));
	for(x=-1; x<=1; x++) for(y=-1; y<=1; y++) for(z=-1; z<=1; z++) {
		m_list.push_back(CObj(RGB(100+x*50,100+y*50,100+z*50),vector_3d(x*6,y*6,z*6),2));
	}
	for(x=-2; x<=2; x++) for(y=-2; y<=2; y++) {
		m_list.push_back(CObj(0,vector_3d(30,x*4+3,y*4+3),1));
	}
}


GLvoid Engine::Init()
{
	glClearColor(0.2f, 0.5f, 0.75f, 1.0f);				//   
	glClearDepth(1.0f);									//        
	glEnable(GL_DEPTH_TEST);							//   
	glShadeModel(GL_SMOOTH);							//   
	glEnable(GL_LINE_SMOOTH);							//   
	glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);				//       
	glEnable(GL_BLEND);									//   , 
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);	//     
	GLfloat light[]={300,400,300,1};
	glLightfv(GL_LIGHT0,GL_POSITION,light);				//     
	glEnable(GL_LIGHT0);								//    
	InitScene();
}

GLvoid Engine::Draw(GLvoid)									
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	//      
	glPushMatrix();										//   
	glRotatef(f1, 1.0, 0.0, 0.0);						//    
	glRotatef(f2, 0.0, 1.0, 0.0);
	glDisable(GL_LIGHTING);								//    
	glutWireCube(CUBE_SIZE);							//      2
	glEnable(GL_LIGHTING);								//    
	GLfloat	diffuse[4]={0,0,0,1};						
	for(int n=0; n<m_list.size(); n++) {
		CObj *obj=&m_list[n];
		glPushMatrix();
		diffuse[0]=(GLfloat)GetRValue(obj->color)/255;
		diffuse[1]=(GLfloat)GetGValue(obj->color)/255;
		diffuse[2]=(GLfloat)GetBValue(obj->color)/255;
		glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,diffuse); 
		glTranslated(obj->offset.x,obj->offset.y,obj->offset.z);
		if(obj->r==1) glutSolidSphere(obj->r,8,8);			//     				
		else glutSolidSphere(obj->r,16,16);			
		glPopMatrix();
	}
	glPopMatrix();
}

GLvoid Engine::Step()
{
	f1+=0.01f;	//  
	f2+=0.03f;
	for(int n=0; n<m_list.size(); n++) {
		CObj &obj=m_list[n];
		obj.offset+=obj.speed;
		CheckCollision(n,&obj);
		CheckBorder(&obj);
	}
}


GLvoid Engine::CheckBorder(CObj* obj)
{
	unsigned int f=0;
	GLfloat g=CUBE_SIZE/2-obj->r;
	if(obj->offset.x>g) f|=1,obj->offset.x=g;
	if(obj->offset.y>g) f|=2,obj->offset.y=g;
	if(obj->offset.z>g) f|=4,obj->offset.z=g;
	if(obj->offset.x<-g) f|=1,obj->offset.x=-g;
	if(obj->offset.y<-g) f|=2,obj->offset.y=-g;
	if(obj->offset.z<-g) f|=4,obj->offset.z=-g;
	if(f&1) obj->speed.x=-obj->speed.x;
	if(f&2) obj->speed.y=-obj->speed.y;
	if(f&4) obj->speed.z=-obj->speed.z;
}

GLvoid Engine::CheckCollision(int index,CObj *obj1)
{
	for(int n=index+1; n<m_list.size(); n++) 
	{
		CObj *obj2=&m_list[n];
		if(obj1->isStand() && obj2->isStand()) return; //     ,   
		vector_3d diff=obj2->offset-obj1->offset;
		double r=obj1->r+obj2->r;						//      
		double d=diff;									//     
		if(d>=r*r) continue;
		d=sqrt(d);
		double s=(r-d)/2;								//   
		diff/=d;
		obj1->offset-=diff*s;							//    
		obj2->offset+=diff*s;
		double m1=obj1->m;
		double m2=obj2->m;
		double v1=obj1->speed*diff;						//     
		double v2=obj2->speed*diff;
		double vv1=(v1*(m1-m2)+v2*m2*2)/(m1+m2);		//  
		double vv2=(v2*(m2-m1)+v1*m1*2)/(m1+m2);
		obj1->speed+=diff*(vv1-v1);						//     
		obj2->speed+=diff*(vv2-v2);
	}
}


GLvoid Engine::Move(int dx,int dy)
{
	f2-=dx;
	f1-=dy;
}

